###############################################################################
#
# Copyright (C) 2013
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

from common import *
from common_dsp import *
from mem_init_file import list_to_hex

# Purpose:
# . Generate the NOF_INPUTS complex subband HEX files for the block generators.
# Description:
# . 

NOF_INPUTS = 24
COMPLEX_WIDTH = 16
NOF_CHANNELS = 64
NOF_TIMESAMPLES = 16
NOF_WORDS_PER_BLOCK =NOF_TIMESAMPLES*NOF_CHANNELS # 1024

MAX_AMPLITUDE = pow(2, COMPLEX_WIDTH/4)-1 # 1/4 of the max
AMPL_INCR = float(MAX_AMPLITUDE)/NOF_CHANNELS
AMPLITUDE = pow(2, COMPLEX_WIDTH/2)-1 # Half of the max
PHASE_SHIFT = 1

PHASE_INCR = 1


MEM_WIDTH  = COMPLEX_WIDTH*2
MEM_DEPTH  = NOF_WORDS_PER_BLOCK

PATH = "../hex"
FILENAME = "complex_subbands"

# ===========================================================================
# Create the lists of complex subbands:
# . 1 deg. phase shift between the inputs
# . Amplitude increases (up to 1/4 of the max) with increasing channel index
# ===========================================================================

#input_list_one_sample = gen_complex_binomials(NOF_INPUTS, AMPLITUDE, PHASE_SHIFT)

#input_lists = []
#for input_nr in range(NOF_INPUTS):
#    input_lists.append( NOF_WORDS_PER_BLOCK*[input_list_one_sample[input_nr]] )

input_lists = []
ampl = 0
for input_nr in range(NOF_INPUTS):
    channel_block = [] # one channel block carries NOF_CHANNELS channel samples
    phase_deg = 0
    for channel_nr in range(NOF_CHANNELS):
        ampl = ampl + AMPL_INCR
        phase_deg = phase_deg + PHASE_INCR
        sample = complex_phasor_to_binomial(ampl, phase_deg)
        channel_block.append(sample)
    input_lists.append(NOF_TIMESAMPLES*channel_block)

# =============================================
# Convert complex type to concatenated unsigned
# =============================================
for input_nr in range(NOF_INPUTS):
    for word in range(NOF_WORDS_PER_BLOCK):
        re = int(round(input_lists[input_nr][word].real))
        im = int(round(input_lists[input_nr][word].imag))
        re_bits = CommonBits(re, COMPLEX_WIDTH)
        im_bits = CommonBits(im, COMPLEX_WIDTH)
        concat_bits = re_bits & im_bits
        input_lists[input_nr][word] = concat_bits.data

# ====================
# Write the HEX files
# ====================
for input_nr in range(NOF_INPUTS):
    list_to_hex( input_lists[input_nr], PATH+"/"+FILENAME+"_"+str(COMPLEX_WIDTH)+'_'+str(input_nr)+".hex", MEM_WIDTH, MEM_DEPTH)

