#! /usr/bin/env python
###############################################################################
#
# Copyright (C) 2013
# ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

# Purpose:
# Description:
# Usage:

COMPLEX_WIDTH = 32
NOF_INPUTS = 10
NOF_OUTPUTS = NOF_INPUTS*(NOF_INPUTS+1)/2

NOF_DUMMY_VISIBILITIES = NOF_INPUTS*NOF_INPUTS-NOF_OUTPUTS
DUMMY_VISIBILITIES = NOF_DUMMY_VISIBILITIES*[0.0]

BUFFER_DEPTH = 256
BUFFER_WIDTH = 2*COMPLEX_WIDTH

from common import *
from common_dsp import *

import test_case
import node_io
import pi_diag_data_buffer

tc = test_case.Testcase('TB - ', '')
io = node_io.NodeIO(tc.nodeImages, tc.base_ip)
db = pi_diag_data_buffer.PiDiagDataBuffer(tc, io, nofStreams=NOF_OUTPUTS, ramSizePerStream=BUFFER_DEPTH)

###############################################################################
# Read the first word from each data buffer (all words per stream are the same)
###############################################################################
data = []
for output_nr in range(NOF_OUTPUTS):
    data.append( db.read_data_buffer(streamNr=output_nr, n=1, radix='uns', width=BUFFER_WIDTH, nofColumns=12) )

###############################################################################
# Convert the unsigned words to complex
###############################################################################
data = flatten(data)
for index,word in enumerate(data):
    word_bits = CommonBits(word, BUFFER_WIDTH)
    re = word_bits[BUFFER_WIDTH-1:COMPLEX_WIDTH]
    im = word_bits[COMPLEX_WIDTH-1:0]
    data[index] = complex(im, re)

###############################################################################
# Convert binomials to complex phasor notation
###############################################################################
for index,word in enumerate(data):
    data[index] = complex_binomial_to_phasor(word)

###############################################################################
# Extract the phases from the complex data
###############################################################################
phases = []
for word in data:
    phases.append(word[1])

################################################################################
# Re-shape the flat list into a matrix and plot it
################################################################################
mat = unique_vis_to_full_matrix(phases)
plot_matrix_color([[mat]])
