-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Testbench for tech_10gbase_r.
-- Description:
--   The tb is not self checking.
--   The tb is used to investigate what clocks the 10gbase_r phy uses.
-- Usage:
--   > as 10
--   > run 100 us

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE work.tech_10gbase_r_component_pkg.ALL;


ENTITY tb_tech_10gbase_r IS
  -- Test bench control parameters
  GENERIC (
    g_technology    : NATURAL := c_tech_select_default;
    g_nof_channels  : NATURAL := 4
  );
END tb_tech_10gbase_r;


ARCHITECTURE tb OF tb_tech_10gbase_r IS

  CONSTANT c_sim               : BOOLEAN:= TRUE;
  CONSTANT phy_loopback_delay  : TIME :=  1 ns;
  
  SIGNAL tr_ref_clk        : STD_LOGIC := '0';
  SIGNAL clk_312           : STD_LOGIC;    -- 312.5 MHz
  SIGNAL clk_156           : STD_LOGIC;    -- 156.25 MHz
  SIGNAL rst_156           : STD_LOGIC;    -- reset for clk_156 clock domain

  -- XGMII interface
  SIGNAL xgmii_tx_dc_arr   : t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL xgmii_rx_dc_arr   : t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);

  -- PHY serial interface
  SIGNAL tx_serial_arr     : STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
  SIGNAL rx_serial_arr     : STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
  
BEGIN

  tr_ref_clk <= NOT tr_ref_clk AFTER tech_10gbase_r_ref_clk_period/2;

  dut : ENTITY work.tech_10gbase_r
  GENERIC MAP (
    g_technology       => g_technology,
    g_sim              => c_sim,
    g_nof_channels     => g_nof_channels
  )
  PORT MAP (
    -- Transceiver PLL reference clock
    tr_ref_clk              => tr_ref_clk,   -- 644.531250 MHz
    
    -- Derived clocks
    clk_312            => clk_312,
    clk_156            => clk_156,
    rst_156            => rst_156,

    -- XGMII interface
    xgmii_tx_dc_arr    => xgmii_tx_dc_arr,
    xgmii_rx_dc_arr    => xgmii_rx_dc_arr,

    -- PHY serial IO
    tx_serial_arr      => tx_serial_arr,
    rx_serial_arr      => rx_serial_arr
  );
  
  -- PHY loopback
  rx_serial_arr <= TRANSPORT tx_serial_arr AFTER phy_loopback_delay;
  
END tb;
