--------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

LIBRARY IEEE, technology_lib, common_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;

ENTITY tech_10gbase_r IS
  GENERIC (
    g_technology     : NATURAL := c_tech_select_default;
    g_sim            : BOOLEAN := FALSE;
    g_nof_channels   : NATURAL := 1
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk              : IN  STD_LOGIC;    -- 644.531250 MHz
    
    -- Derived clocks
    clk_312                 : OUT STD_LOGIC;    -- 312.5 MHz
    clk_156                 : OUT STD_LOGIC;    -- 156.25 MHz
    rst_156                 : OUT STD_LOGIC;    -- reset for clk_156_in clock domain

    -- XGMII interface
    xgmii_tx_dc_arr         : IN  t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);  -- 72 bit
    xgmii_rx_dc_arr         : OUT t_xgmii_dc_arr(g_nof_channels-1 DOWNTO 0);  -- 72 bit

    -- PHY serial IO
    tx_serial_arr           : OUT STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
    rx_serial_arr           : IN  STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0)
  );
END tech_10gbase_r;


ARCHITECTURE str OF tech_10gbase_r IS  

  SIGNAL clk_156_out        : STD_LOGIC;    -- connect to clk_156_out to clk_156_in to avoid delta-cycle differences in simulation
  
BEGIN
 
  clk_156 <= clk_156_out;
  
  gen_ip_arria10 : IF g_technology=c_tech_arria10 GENERATE
    u0 : ENTITY work.tech_10gbase_r_arria10
    GENERIC MAP (g_sim, g_nof_channels)
    PORT MAP (tr_ref_clk,
              clk_312, clk_156_out, clk_156_out, rst_156,
              xgmii_tx_dc_arr, xgmii_rx_dc_arr,
              tx_serial_arr, rx_serial_arr);
  END GENERATE;
      
END str;

